﻿/*
    Little Registry Optimizer
    Copyright (C) 2008-2009 Little Apps (http://www.littleapps.co.cc/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.IO;
using Microsoft.Win32;
using System.Text.RegularExpressions;
using System.Threading;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace Little_Registry_Optimizer
{
    public class Hive : IDisposable
    {
        private readonly string strHiveName, strHivePath;

        private volatile string strRootKey, strKeyName;

        private volatile string strOldHivePath = Utils.GetTempHivePath();
        public string OldHivePath
        {
            get { return strOldHivePath; }
        }

        private volatile string strNewHivePath = Utils.GetTempHivePath();
        public string NewHivePath
        {
            get { return strNewHivePath; }
        }

        private readonly FileInfo _fi = null;
        public FileInfo HiveFileInfo
        {
            get { return _fi; }
        }

        private readonly long lOldHiveSize = 0;
        public long OldHiveSize
        {
            get { return lOldHiveSize; }
        }

        private volatile uint lNewHiveSize = 0;
        public long NewHiveSize
        {
            get { return lNewHiveSize; }
        }

        public volatile bool bAnaylzed = false, bCompacted = false;

        private volatile int hKey = 0;
        private bool disposed = false;

        /// <summary>
        /// Constructor for Hive class
        /// </summary>
        /// <param name="HiveName">Name of Hive (\REGISTRY\USER\...)</param>
        /// <param name="HivePath">Path to Hive (\Device\HarddiskVolumeX\Windows\System32\config\... or C:\Windows\System32\config\...)</param>
        public Hive(string HiveName, string HivePath)
        {
            this.strHiveName = HiveName;
            if (File.Exists(HivePath))
                this.strHivePath = HivePath;
            else
                this.strHivePath = Utils.ConvertDeviceToMSDOSName(HivePath);

            try
            {
                this._fi = new FileInfo(this.strHivePath);
                this.lOldHiveSize = Utils.GetFileSize(this.strHivePath);
            }
            catch { System.Diagnostics.Debug.WriteLine("error opening registry hive"); }
        }

        public void Dispose()
        {
            if (!this.disposed)
            {
                if (this.hKey != 0)
                    PInvoke.RegCloseKey(hKey);

                hKey = 0;

                if (File.Exists(strOldHivePath))
                    File.Delete(strOldHivePath);

                disposed = true;
            }

            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Uses Windows RegSaveKeyA API to rewrite registry hive
        /// </summary>
        public void AnalyzeHive()
        {
            try
            {
                if (this.bAnaylzed)
                    throw new Exception("Hive cannot already be analyzed");

                int nRet = 0, hkey = 0;

                this.strRootKey = this.strHiveName.ToLower();
                this.strKeyName = strRootKey.Substring(strRootKey.LastIndexOf('\\') + 1);

                // Open Handle to registry key
                if (strRootKey.StartsWith(@"\registry\machine"))
                    nRet = PInvoke.RegOpenKeyA((uint)PInvoke.HKEY.HKEY_LOCAL_MACHINE, strKeyName, ref hkey);
                if (strRootKey.StartsWith(@"\registry\user"))
                    nRet = PInvoke.RegOpenKeyA((uint)PInvoke.HKEY.HKEY_USERS, strKeyName, ref hkey);

                if (nRet != 0)
                    return;

                this.hKey = hkey;

                // Begin Critical Region
                Thread.BeginCriticalRegion();

                // Flush hive key
                PInvoke.RegFlushKey(hkey);

                // Function will fail if file already exists
                if (File.Exists(this.strNewHivePath))
                    File.Delete(this.strNewHivePath);

                // Use API to rewrite the registry hive
                nRet = PInvoke.RegSaveKeyA(hkey, this.strNewHivePath, 0);

                this.lNewHiveSize = (uint)Utils.GetFileSize(this.strNewHivePath);

                if (File.Exists(this.strNewHivePath))
                    this.bAnaylzed = true;

                // End Critical Region
                Thread.EndCriticalRegion();
            }
            catch
            {
                System.Diagnostics.Debug.WriteLine("error analyzing registry hive");
            }
        }

        /// <summary>
        /// Compacts the registry hive
        /// </summary>
        public void CompactHive() 
        {
            if (this.bAnaylzed == false || this.hKey <= 0 || !File.Exists(this.strNewHivePath))
                throw new Exception("You must analyze the hive before you can compact it");

            if (this.bCompacted)
                throw new Exception("The hive has already been compacted");

            // Begin Critical Region
            Thread.BeginCriticalRegion();

            // Old hive cant exist or function will fail
            if (File.Exists(this.strOldHivePath))
                File.Delete(this.strOldHivePath);

            // Replace hive with compressed hive
            //if (PInvoke.RegReplaceKeyA(this.hKey, null, this.strNewHivePath, this.strOldHivePath) != 0)
            //throw Marshal.GetExceptionForHR(Marshal.GetLastWin32Error());
            //System.Windows.Forms.MessageBox.Show("Hive: " + this.ToString() + " Error Code: " + Marshal.GetLastWin32Error());

            int ret = PInvoke.RegReplaceKeyA(this.hKey, null, this.strNewHivePath, this.strOldHivePath);
            if (ret != 0)
            {
                System.Windows.Forms.MessageBox.Show("Hive: " + this.ToString() + " Error Code: " + ret);
                return;
            }

            // Hive should now be replaced with temporary hive
            PInvoke.RegCloseKey(this.hKey);

            // End Critical Region
            Thread.EndCriticalRegion();

            this.bCompacted = true;
        }

        /// <summary>
        /// Returns the filename of the hive
        /// </summary>
        /// <returns>Hive filename</returns>
        public override string ToString()
        {
            return (string)_fi.Name.Clone();
        }
    }

    //public class HiveCollection : List<Hive>
    //{
    //    public Hive this[int index]
    //    {
    //        get { return (Hive)this[index]; }
    //        set { this[index] = value; }
    //    }

    //    public int Add(Hive h)
    //    {
    //        if (h == null)
    //            throw new ArgumentNullException("Hive");

    //        return (this.Add(h));
    //    }

    //    public void Remove(Hive h)
    //    {
    //        this.Remove(h);
    //    }

    //    public int IndexOf(Hive h)
    //    {
    //        return (this.IndexOf(h));
    //    }

    //    public bool Contains(Hive h)
    //    {
    //        return (this.Contains(h));
    //    }
    //}
}
