﻿/*
    Little Registry Optimizer
    Copyright (C) 2008-2009 Little Apps (http://www.littleapps.co.cc/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Threading;
using System.Runtime.InteropServices;
using System.IO;
using System.Windows.Forms;
using Microsoft.Win32;
using System.Reflection;

namespace Little_Registry_Optimizer
{
    public partial class Main : Form
    {

        public static List<Hive> arrHives = new List<Hive>();
        private bool bAllowFormClosed = true;

        /// <summary>
        /// Returns the last time the program started
        /// </summary>
        public static DateTime LastStart
        {
            get
            {
                if (Properties.Settings.Default.objLastStart != 0)
                    return DateTime.FromBinary(Properties.Settings.Default.objLastStart);
                else
                    return DateTime.MinValue;
            }
            set
            {
                DateTime dt = value;
                Properties.Settings.Default.objLastStart = dt.ToBinary();
            }
        }

        /// <summary>
        /// Returns the last time the program performed a scan
        /// </summary>
        public static DateTime LastScan
        {
            get 
            {
                if (Properties.Settings.Default.objLastScan != 0)
                    return DateTime.FromBinary(Properties.Settings.Default.objLastScan);
                else
                    return DateTime.MinValue;
            }
            set
            {
                DateTime dt = value;
                Properties.Settings.Default.objLastScan = dt.ToBinary(); 
            }
        }

        /// <summary>
        /// The last time a update check was performed
        /// </summary>
        public static DateTime LastUpdateCheck
        {
            get {
                if (Properties.Settings.Default.objLastUpdate != 0)
                    return DateTime.FromBinary(Properties.Settings.Default.objLastUpdate);
                else
                    return DateTime.MinValue;
            }
            set {
                DateTime dt = value;
                Properties.Settings.Default.objLastUpdate = dt.ToBinary(); 
            }
        }

        /// <summary>
        /// Returns the build time of the executable (only returns the date)
        /// </summary>
        public static DateTime BuildTime
        {
            get
            {
                return new DateTime(2000, 1, 1).AddDays(Assembly.GetExecutingAssembly().GetName().Version.Build);
            }
        }

        public Main()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            // Display splash screen on different thread
            Thread threadSplash = new Thread(new ThreadStart(SplashScreen.ShowSplashScreen));
            threadSplash.SetApartmentState(ApartmentState.STA);
            threadSplash.Start();
            threadSplash.Join();

            // Give this form focus
            this.Activate();

            // Update list view
            PopulateListView();

            // Display time from last scan
            if (Main.LastScan != DateTime.MinValue)
                this.labelStatus.Text = "Last Scanned On " + LastScan.ToShortDateString();
            else
                this.labelStatus.Text = "Your registry needs compacting!";

            // Configure progress bars
            this.progressBarAnalyzed.Minimum = 0;
            this.progressBarAnalyzed.Maximum = arrHives.Count;

            this.progressBarDefrag.Minimum = 0;
            this.progressBarDefrag.Maximum = arrHives.Count;

            base.OnLoad(e);
        }

        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            if (!this.bAllowFormClosed)
            {
                MessageBox.Show(this, "Please wait for the current operation to finish", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                e.Cancel = true;
            }

            if (MessageBox.Show(this, "Are you sure?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                e.Cancel = true;

            if (!e.Cancel)
                Main.LastStart = DateTime.Now.Date;

            base.OnClosing(e);
        }

        /// <summary>
        /// Populates list view with hive collection
        /// </summary>
        private void PopulateListView()
        {
            long lTotalOldSize = 0, lTotalNewSize = 0;

            this.listView1.Items.Clear();

            foreach (Hive h in arrHives)
            {
                string strHiveFileName = h.HiveFileInfo.Name;
                lTotalOldSize += h.OldHiveSize;
                if (h.NewHiveSize > 0)
                    lTotalNewSize += h.NewHiveSize;

                ListViewItem lvi = new ListViewItem(new string[] { h.HiveFileInfo.Name, Utils.ConvertSizeToString(h.OldHiveSize), Utils.ConvertSizeToString(h.NewHiveSize) });
                this.listView1.Items.Add(lvi);
            }

            if (lTotalNewSize > 0 && lTotalOldSize > 0)
            {
                ListViewItem lvi = new ListViewItem(new string[] { "TOTAL", Utils.ConvertSizeToString(lTotalOldSize), Utils.ConvertSizeToString(lTotalNewSize) });
                lvi.Font = new Font(FontFamily.GenericSerif, 8.25F, FontStyle.Bold);
                this.listView1.Items.Add(lvi);
            }

            this.listView1.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize);
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About frmAbout = new About();
            frmAbout.ShowDialog(this);
        }

        private void buttonAnalyze_Click(object sender, EventArgs e)
        {
            Thread threadCurrent;
            string strMsg =
                "You must close running programs before optimizing the registry.\n" +
                "Please save your work and close any running programs now.";

            if (MessageBox.Show(this, strMsg, Application.ProductName, MessageBoxButtons.OKCancel, MessageBoxIcon.Warning) != DialogResult.OK)
                return;

            SetButtonsEnabled(false);

            if (arrHives.Count > 0)
            {
                foreach (Hive h in Main.arrHives)
                {
                    // Analyze Hive
                    threadCurrent = new Thread(new ThreadStart(h.AnalyzeHive));
                    threadCurrent.Start();
                    threadCurrent.Join();

                    // Update list view
                    PopulateListView();

                    // Update progress bar
                    this.progressBarAnalyzed.Value++;
                    this.labelStatus.Text = string.Format(@"Analyzed: {0}/{1}", this.progressBarAnalyzed.Value, arrHives.Count);
                }
            }

            SetButtonsEnabled(true);

            // Save current date
            LastScan = DateTime.Now.Date;
        }

        private void SetButtonsEnabled(bool bValue)
        {
            this.buttonStart.Enabled = bValue;
            this.bAllowFormClosed = bValue;
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            long lSeqNum = 0;
            Thread threadCurrent;

            if (MessageBox.Show(this, "Are you sure you want to compact your registry?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                return;

            this.progressBarDefrag.Minimum = 0;
            this.progressBarDefrag.Maximum = arrHives.Count;
            this.labelStatus.Text = string.Format(@"Optimized: 0/{0}", arrHives.Count);

            SysRestore.StartRestore("Before Little Registry Cleaner Optimization", SysRestore.RestoreType.Restore, out lSeqNum);

            SetButtonsEnabled(false);

            foreach (Hive h in Main.arrHives)
            {
                threadCurrent = new Thread(new ThreadStart(h.CompactHive));
                threadCurrent.Start();
                threadCurrent.Join();

                this.progressBarDefrag.Value++;
                this.labelStatus.Text = string.Format(@"Optimized: {0}/{1}", this.progressBarDefrag.Value, arrHives.Count);
            }

            SetButtonsEnabled(true);

            SysRestore.EndRestore(lSeqNum);

            if (MessageBox.Show(this, "You must restart your computer before the new setting will take effect. Do you want to restart your computer now?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                PInvoke.ExitWindowsEx(0x02, PInvoke.MajorOperatingSystem | PInvoke.MinorReconfig | PInvoke.FlagPlanned);
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Options frmOptions = new Options();
            frmOptions.Show(this);
        }
    }
}
