﻿/*
    Little Registry Optimizer
    Copyright (C) 2008-2009 Little Apps (http://www.littleapps.co.cc/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.IO.Packaging;
using System.Xml;
using System.Windows.Forms;
using Microsoft.Win32;

namespace Little_Registry_Optimizer
{
    static class Utils
    {
        [DllImport("kernel32.dll")]
        public static extern uint QueryDosDevice([In, Optional] string lpDeviceName, [Out] StringBuilder lpTargetPath, [In] int ucchMax);

        /// <summary>
        /// Converts the size in bytes to a formatted string
        /// </summary>
        /// <param name="Length">Size in bytes</param>
        /// <returns>Formatted String</returns>
        public static string ConvertSizeToString(long Length)
        {
            if (Length <= 0)
                return "Unknown";

            float nSize;
            string strSizeFmt, strUnit = "";

            if (Length < 1000)             // 1KB
            {
                nSize = Length;
                strUnit = " B";
            }
            else if (Length < 1000000)     // 1MB
            {
                nSize = Length / (float)0x400;
                strUnit = " KB";
            }
            else if (Length < 1000000000)   // 1GB
            {
                nSize = Length / (float)0x100000;
                strUnit = " MB";
            }
            else
            {
                nSize = Length / (float)0x40000000;
                strUnit = " GB";
            }

            if (nSize == (int)nSize)
                strSizeFmt = nSize.ToString("0");
            else if (nSize < 10)
                strSizeFmt = nSize.ToString("0.00");
            else if (nSize < 100)
                strSizeFmt = nSize.ToString("0.0");
            else
                strSizeFmt = nSize.ToString("0");

            return strSizeFmt + strUnit;
        }

        /// <summary>
        /// Converts \\Device\\HarddiskVolumeX\... to X:\...
        /// </summary>
        /// <param name="DevicePath">Device name with path</param>
        /// <returns>Drive path</returns>
        public static string ConvertDeviceToMSDOSName(string DevicePath)
        {
            string strDevicePath = string.Copy(DevicePath.ToLower());
            string strRetVal = "";

            // Convert \Device\HarddiskVolumeX\... to X:\...
            foreach (KeyValuePair<string, string> kvp in QueryDosDevice())
            {
                string strDrivePath = kvp.Key.ToLower();
                string strDeviceName = kvp.Value.ToLower();

                if (strDevicePath.StartsWith(strDeviceName))
                {
                    strRetVal = strDevicePath.Replace(strDeviceName, strDrivePath);
                    break;
                }
            }

            return strRetVal;
        }

        private static Dictionary<string, string> QueryDosDevice()
        {
            Dictionary<string, string> ret = new Dictionary<string, string>();

            foreach (DriveInfo di in DriveInfo.GetDrives())
            {
                if (di.IsReady)
                {
                    string strDrivePath = di.Name.Substring(0, 2);
                    StringBuilder strDeviceName = new StringBuilder(260);

                    // Convert C: to \Device\HarddiskVolume1
                    if (QueryDosDevice(strDrivePath, strDeviceName, 260) > 0)
                        ret.Add(strDrivePath, strDeviceName.ToString());
                }
            }

            return ret;
        }

        /// <summary>
        /// Gets a temporary path for a registry hive
        /// </summary>
        public static string GetTempHivePath()
        {
            try
            {
                string tempPath = Path.Combine(Path.GetTempPath(), Path.GetRandomFileName());

                // File cant exists, keep retrying until we get a file that doesnt exist
                if (File.Exists(tempPath))
                    return GetTempHivePath();

                return tempPath;
            }
            catch (IOException)
            {
                return GetTempHivePath();
            }
        }

        /// <summary>
        /// Gets the file size
        /// </summary>
        /// <param name="filePath">Path to the filename</param>
        /// <returns>File Size</returns>
        public static long GetFileSize(string filePath)
        {
            try
            {
                FileInfo fi = new FileInfo(filePath);

                return fi.Length;
            }
            catch (Exception)
            {
                return 0;
            }
        }

        [Obsolete("Use system restore instead", true)]
        public static void BackupHives()
        {
            string strZipPath = string.Format("{0}\\{1:yyyy}_{1:MM}_{1:dd}_{1:HH}{1:mm}.dat", Properties.Settings.Default.strBackupFolder, DateTime.Now);

            Package zip = ZipPackage.Open(strZipPath, FileMode.OpenOrCreate, FileAccess.ReadWrite);

            foreach (Hive h in Main.arrHives)
            {
                if (h.bCompacted)
                    AddToArchive(zip, h.OldHivePath, h.HiveFileInfo.Name);
            }

            zip.Close();
        }

        [Obsolete]
        private static void AddToArchive(Package zip, string FilePath, string FileName)
        {
            // Add forward slash to filename and replace spaces with underscore
            string strZipUri = string.Concat("/", FileName.Replace(" ", "_"));

            Uri uriPart = new Uri(strZipUri, UriKind.Relative);

            PackagePart pkgPart = zip.CreatePart(uriPart, System.Net.Mime.MediaTypeNames.Application.Zip, CompressionOption.Maximum);

            using (StreamWriter stream = new StreamWriter(pkgPart.GetStream()))
            {
                byte[] arr = File.ReadAllBytes(FilePath);

                stream.Write(arr);
            }

            return;
        }

        /// <summary>
        /// Connects to update server and sees if update is available
        /// </summary>
        /// <param name="strVersion">Current version available</param>
        /// <param name="strReleaseDate">Date version was released</param>
        /// <param name="strChangeLogURL">Changelog URL</param>
        /// <param name="strDownloadURL">Download URL</param>
        /// <returns>True if a update is available</returns>
        public static bool FindUpdate(ref string strVersion, ref string strReleaseDate, ref string strChangeLogURL, ref string strDownloadURL)
        {
            bool bRet = false;

            try
            {
                XmlReader xmlReader = XmlTextReader.Create(Properties.Settings.Default.strUpdateURL);

                while (xmlReader.Read())
                {
                    xmlReader.MoveToContent();

                    if (xmlReader.NodeType == XmlNodeType.Element)
                    {
                        if (xmlReader.Name.CompareTo("version") == 0)
                            strVersion = xmlReader.ReadString();
                        else if (xmlReader.Name.CompareTo("date") == 0)
                            strReleaseDate = xmlReader.ReadString();
                        else if (xmlReader.Name.CompareTo("changelog") == 0)
                            strChangeLogURL = xmlReader.ReadString();
                        else if (xmlReader.Name.CompareTo("download") == 0)
                            strDownloadURL = xmlReader.ReadString();
                    }
                }

                Version verApp = new Version(Application.ProductVersion);
                Version verLatest = new Version(strVersion);

                // Compare major and minor version parts
                if (verApp.Major < verLatest.Major || verApp.Minor < verLatest.Minor)
                    bRet = true;

                DateTime dtReleaseDate, dtBuildDate = Main.BuildTime;

                if (DateTime.TryParse(strReleaseDate, out dtReleaseDate))
                {
                    // See if date from xml is later than build date
                    if (DateTime.Compare(dtReleaseDate, dtBuildDate) > 0)
                        bRet = true;
                }

            }
            catch (System.Net.WebException ex)
            {
                if (MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.RetryCancel, MessageBoxIcon.Error) == DialogResult.Retry)
                    return FindUpdate(ref strVersion, ref strReleaseDate, ref strChangeLogURL, ref strDownloadURL);
            }

            return bRet;
        }

        /// <summary>
        /// Checks for default program then launches URI
        /// </summary>
        /// <param name="WebAddress">The address to launch</param>
        /// <returns>False if the scheme is not set</returns>
        public static void LaunchURI(string WebAddress)
        {
            Help.ShowHelp(Form.ActiveForm, string.Copy(WebAddress));
        }

    }
}
